import subprocess
import unittest


class TestDataIsInstalled(unittest.TestCase):
    TEST_CMD = "python -c 'from vulndb import DBVuln; DBVuln.from_id(123)'"

    def test_data_is_installed_in_virtualenv(self):
        # When we run this in the current CMD it will load the python class
        # and db files from this directory (because of python's PATH)
        subprocess.check_output(self.TEST_CMD, shell=True)

        subprocess.check_output('python setup.py install', shell=True)
        # Now we run it in /tmp , where there is no vulndb in current PATH
        # so it will try to find it inside the site-packages
        subprocess.check_output(self.TEST_CMD, shell=True, cwd='/tmp/')



WASC_URL = 'http://projects.webappsec.org/w/page'

WASC_ID_TO_URL = {42: WASC_URL + '/13246913/Abuse%20of%20Functionality',
                  11: WASC_URL + '/13246915/Brute%20Force',
                  7:  WASC_URL + '/13246916/Buffer%20Overflow',
                  12: WASC_URL + '/13246917/Content%20Spoofing',
                  18: WASC_URL + '/13246918/Credential%20and%20Session%20Prediction',
                  8:  WASC_URL + '/13246920/Cross%20Site%20Scripting',
                  9:  WASC_URL + '/13246919/Cross%20Site%20Request%20Forgery',
                  10: WASC_URL + '/13246921/Denial%20of%20Service',
                  45: WASC_URL + '/13246925/Fingerprinting',
                  27: WASC_URL + '/13246930/HTTP%20Response%20Smuggling',
                  25: WASC_URL + '/13246931/HTTP%20Response%20Splitting',
                  26: WASC_URL + '/13246928/HTTP%20Request%20Smuggling',
                  24: WASC_URL + '/13246929/HTTP%20Request%20Splitting',
                  3:  WASC_URL + '/13246946/Integer%20Overflows',
                  6:  WASC_URL + '/13246926/Format%20String',
                  29: WASC_URL + '/13246947/LDAP%20Injection',
                  30: WASC_URL + '/13246948/Mail%20Command%20Injection',
                  28: WASC_URL + '/13246949/Null%20Byte%20Injection',
                  31: WASC_URL + '/13246950/OS%20Commanding',
                  33: WASC_URL + '/13246952/Path%20Traversal',
                  34: WASC_URL + '/13246953/Predictable%20Resource%20Location',
                  32: WASC_URL + '/13246956/Routing%20Detour',
                  37: WASC_URL + '/13246960/Session%20Fixation',
                  35: WASC_URL + '/13246962/SOAP%20Array%20Abuse',
                  36: WASC_URL + '/13246964/SSI%20Injection',
                  19: WASC_URL + '/13246963/SQL%20Injection',
                  38: WASC_URL + '/13246981/URL%20Redirector%20Abuse',
                  39: WASC_URL + '/13247005/XPath%20Injection',
                  41: WASC_URL + '/13247001/XML%20Attribute%20Blowup',
                  43: WASC_URL + '/13247003/XML%20External%20Entities',
                  44: WASC_URL + '/13247002/XML%20Entity%20Expansion',
                  23: WASC_URL + '/13247004/XML%20Injection',
                  46: WASC_URL + '/13247006/XQuery%20Injection',
                  15: WASC_URL + '/13246914/Application%20Misconfiguration',
                  16: WASC_URL + '/13246922/Directory%20Indexing',
                  17: WASC_URL + '/13246932/Improper%20Filesystem%20Permissions',
                  20: WASC_URL + '/13246933/Improper%20Input%20Handling',
                  22: WASC_URL + '/13246934/Improper%20Output%20Handling',
                  13: WASC_URL + '/13246936/Information%20Leakage',
                  48: WASC_URL + '/13246937/Insecure%20Indexing',
                  21: WASC_URL + '/13246938/Insufficient%20Anti-automation',
                  1:  WASC_URL + '/13246939/Insufficient%20Authentication',
                  2:  WASC_URL + '/13246940/Insufficient%20Authorization',
                  49: WASC_URL + '/13246942/Insufficient%20Password%20Recovery',
                  40: WASC_URL + '/13246943/Insufficient%20Process%20Validation',
                  47: WASC_URL + '/13246944/Insufficient%20Session%20Expiration',
                  4:  WASC_URL + '/13246945/Insufficient%20Transport%20Layer%20Protection',
                  14: WASC_URL + '/13246959/Server%20Misconfiguration'
                  }



import os
import json

from vulndb.constants.wasc import WASC_ID_TO_URL
from vulndb.constants.cwe import CWE_URL_FMT
from vulndb.constants.owasp import (OWASP_TOP10_2010_URL_FMT,
                                    OWASP_TOP10_2013_URL_FMT)


class DBVuln(object):
    """
    Wrapper around a vulnerability as defined in vulndb JSON files.

    :see: https://github.com/vulndb/data/issues/5
    :see: https://github.com/vulndb/data
    """
    DB_PATH = os.path.join(os.path.dirname(os.path.realpath(__file__)), 'db')

    def __init__(self, _id=None, title=None, description=None, severity=None,
                 wasc=None, tags=None, cwe=None, owasp_top_10=None, fix=None,
                 references=None):
        """
        Creates a new DBVuln, setting the internal attributes to the provided
        kwargs.
        """
        self.id = _id
        self.title = title
        self.description = description
        self.severity = severity
        self.wasc = wasc
        self.tags = tags
        self.cwe = cwe
        self.owasp_top_10 = owasp_top_10
        self.fix = fix
        self.references = references

    @classmethod
    def from_file(cls, db_file):
        """
        This is an alternative "constructor" for the DBVuln class which loads
        the data from a file.
        """
        data = DBVuln.load_from_json(db_file)
        return cls(**data)

    @classmethod
    def from_id(cls, _id):
        """
        This is an alternative "constructor" for the DBVuln class which searches
        the db directory to find the right file for the provided _id
        """
        db_file = DBVuln.get_file_for_id(_id)
        data = DBVuln.load_from_json(db_file)
        return cls(**data)

    @staticmethod
    def get_file_for_id(_id):
        """
        Given _id, search the DB for the file which contains the data
        :param _id: The id to search (int)
        :return: The filename
        """
        file_start = '%s-' % _id

        for _file in os.listdir(DBVuln.DB_PATH):
            if _file.startswith(file_start):
                return os.path.join(DBVuln.DB_PATH, _file)

        raise NotFoundException('No data for ID %s' % _id)

    @staticmethod
    def load_from_json(db_file):
        """
        Parses the JSON data and returns it

        :param db_file: File and path pointing to the JSON file to parse
        :raises: All kind of exceptions if the file doesn't exist or JSON is
                 invalid.
        :return: None
        """
        # There are a couple of things I don't do here, and are on purpose:
        #   - I want to fail if the file doesn't exist
        #   - I want to fail if the file doesn't contain valid JSON
        raw_data = json.loads(file(db_file).read())

        # Here I don't do any error handling either, I expect the JSON files to
        # be valid
        data = {
            '_id': raw_data['id'],
            'title': raw_data['title'],
            'description': DBVuln.handle_multiline_field(raw_data['description']),
            'severity': raw_data['severity'],
            'wasc': raw_data.get('wasc', None),
            'tags': raw_data['tags'],
            'cwe': raw_data.get('cwe', None),
            'owasp_top_10': raw_data.get('owasp_top_10', None),
            'fix': raw_data['fix'],
            'references': DBVuln.handle_references(raw_data.get('references', [])),
        }

        return data

    @property
    def fix_guidance(self):
        """
        :return: The text associated with the fix guidance:

            "fix": {
                    "guidance": "A very long text explaining how to fix...",
                    "effort": 50
                    },
        """
        return self.handle_multiline_field(self.fix['guidance'])

    @property
    def fix_effort(self):
        """
        :return: The effort (in minutes) associated with the fix

            "fix": {
                    "guidance": "A very long text explaining how to fix...",
                    "effort": 50
                    },
        """
        return self.fix['effort']

    def get_wasc_url(self, wasc_id):
        """
        :return: The URL associated with the wasc_id, usually the WASC ID is
                 received by the developer from self.wasc and he uses this
                 method to give the user a URL

                 None is returned if the URL can't be found
        """
        if wasc_id in WASC_ID_TO_URL:
            return WASC_ID_TO_URL[wasc_id]

    def get_cwe_url(self, cwe_id):
        """
        Similar to get_wasc_url() but for CWE
        """
        return CWE_URL_FMT % cwe_id

    def get_owasp_top_10_url(self, owasp_version, risk_id):
        """
        Similar to get_wasc_url() but for OWASP Top 10
        """
        owasp_version = int(owasp_version)

        if owasp_version == 2010:
            return OWASP_TOP10_2010_URL_FMT % risk_id
        elif owasp_version == 2013:
            return OWASP_TOP10_2013_URL_FMT % risk_id

    @staticmethod
    def handle_multiline_field(field_data):
        """
        According to the spec there might be some fields which contain long
        descriptions, which might be strings or lists with strings. I translate
        the list of strings into a long string and return it.

        :see: https://github.com/vulndb/data/issues/5
        :param field_data: A string or a list
        :return: A string
        """
        if isinstance(field_data, basestring):
            return field_data

        return ''.join(field_data)

    @staticmethod
    def handle_references(json_references):
        """
        Create a list of reference objects that represent this part of the JSON
        data:

          "references": [
              {"url": "http://foo.com/xss", "title": "First reference to XSS vulnerability"},
              {"url": "http://asp.net/xss", "title": "How to fix XSS vulns in ASP.NET"},
              {"url": "http://owasp.org/xss", "title": "OWASP desc for XSS"}
            ]

        :return: A list with Reference objects
        """
        reference_list = []
        for reference_dict in json_references:
            reference_list.append(Reference(reference_dict['url'],
                                            reference_dict['title']))

        return reference_list

    def __str__(self):
        return 'DBVulnerability for %s - %s' % (self.title, self.id)

    def __repr__(self):
        return '<DBVulnerability (title: "%s" | id: %s) >' % (self.title,
                                                              self.id)

    def __eq__(self, other):
        return self.id == other.id and \
               self.title == other.title and \
               self.description == other.description and \
               self.severity == other.severity and \
               self.wasc == other.wasc and \
               self.tags == other.tags and \
               self.cwe == other.cwe and \
               self.owasp_top_10 == other.owasp_top_10 and \
               self.fix == other.fix and \
               self.references == other.references


class Reference(object):
    def __init__(self, url, title):
        self.url = url
        self.title = title

    def __str__(self):
        return '[%s](%s)' % (self.title, self.url)

    def __repr__(self):
        return '<Reference (%s|%s)>' % (self.title, self.url)

    def __eq__(self, other):
        return self.url == other.url and self.title == other.title


class NotFoundException(Exception):
    pass


import unittest
import os

from vulndb import DBVuln
from vulndb.db_vuln import Reference

MOCK_TITLE = 'Title'
MOCK_DESC = 'Description for the vulnerability'
MOCK_ID = 123
MOCK_SEVERITY = 'medium'
MOCK_WASC = ['2']
MOCK_TAGS = ['hello', 'world']
MOCK_CWE = ['89']
MOCK_OWASP_TOP_10 = {"2010": [1], "2013": [2]}
MOCK_FIX = {"guidance": "A very long text explaining how to fix...",
            "effort": 50}
MOCK_REFERENCES = [{"url": "http://foo.com/xss", "title": "First reference to XSS vulnerability"},
                   {"url": "http://asp.net/xss", "title": "How to fix XSS vulns in ASP.NET"}]


class TestDBVuln(unittest.TestCase):
    DEFAULT_KWARGS = {
            '_id': MOCK_ID,
            'title': MOCK_TITLE,
            'description': MOCK_DESC,
            'severity': MOCK_SEVERITY,
            'wasc': MOCK_WASC,
            'tags': MOCK_TAGS,
            'cwe': MOCK_CWE,
            'owasp_top_10': MOCK_OWASP_TOP_10,
            'fix': MOCK_FIX,
            'references': MOCK_REFERENCES,
        }

    def setUp(self):
        self.original_db_path = DBVuln.DB_PATH

        current_dir = os.path.dirname(os.path.realpath(__file__))
        DBVuln.DB_PATH = os.path.join(current_dir, 'db')

    def tearDown(self):
        DBVuln.DB_PATH = self.original_db_path

    def test_basic(self):
        dbv = DBVuln(**self.DEFAULT_KWARGS)

        self.assertEqual(dbv.title, MOCK_TITLE)
        self.assertEqual(dbv.description, MOCK_DESC)
        self.assertEqual(dbv.id, MOCK_ID)
        self.assertEqual(dbv.severity, MOCK_SEVERITY)
        self.assertEqual(dbv.wasc, MOCK_WASC)
        self.assertEqual(dbv.tags, MOCK_TAGS)
        self.assertEqual(dbv.cwe, MOCK_CWE)
        self.assertEqual(dbv.owasp_top_10, MOCK_OWASP_TOP_10)
        self.assertEqual(dbv.fix, MOCK_FIX)
        self.assertEqual(dbv.references, MOCK_REFERENCES)

    def test_from_file(self):
        _file = os.path.join(DBVuln.DB_PATH, '123-spec-example.json')

        dbv_1 = DBVuln.from_file(_file)
        dbv_2 = DBVuln.from_id(123)

        self.assertEqual(dbv_1, dbv_2)

    def test_from_id(self):
        dbv = DBVuln.from_id(123)

        expected_references = [Reference("http://foo.com/xss",
                                         "First reference to XSS vulnerability"),
                               Reference("http://asp.net/xss",
                                         "How to fix XSS vulns in ASP.NET")]

        self.assertEqual(dbv.title, u'Cross-Site Scripting')
        self.assertEqual(dbv.description, u'A very long description for'
                                          u' Cross-Site Scripting')
        self.assertEqual(dbv.id, MOCK_ID)
        self.assertEqual(dbv.severity, MOCK_SEVERITY)
        self.assertEqual(dbv.wasc, [u'0003'])
        self.assertEqual(dbv.tags, [u'xss', u'client side'])
        self.assertEqual(dbv.cwe, [u'0003', u'0007'])
        self.assertEqual(dbv.owasp_top_10, {"2010": [1], "2013": [2]},)
        self.assertEqual(dbv.fix, {u"guidance": u"A very long text explaining"
                                                u" how to fix XSS"
                                                u" vulnerabilities",
                                   u"effort": 50})
        self.assertEqual(dbv.references, expected_references)
        self.assertEqual(dbv.fix_effort, 50)
        self.assertEqual(dbv.fix_guidance, u"A very long text explaining"
                                           u" how to fix XSS vulnerabilities")

    def test_get_cwe_url(self):
        dbv = DBVuln(**self.DEFAULT_KWARGS)
        self.assertEqual(dbv.get_cwe_url(89),
                         'https://cwe.mitre.org/data/definitions/89.html')

    def test_get_wasc_url(self):
        dbv = DBVuln(**self.DEFAULT_KWARGS)
        self.assertEqual(dbv.get_wasc_url(3),
                         'http://projects.webappsec.org/w/page/13246946/Integer%20Overflows')

    def test_get_owasp_top_10_url(self):
        dbv = DBVuln(**self.DEFAULT_KWARGS)
        self.assertEqual(dbv.get_owasp_top_10_url(2010, 2),
                         'https://www.owasp.org/index.php/Top_10_2010-A2')

        self.assertEqual(dbv.get_owasp_top_10_url(2013, 2),
                         'https://www.owasp.org/index.php/Top_10_2013-A2')

        self.assertEqual(dbv.get_owasp_top_10_url(2033, 2), None)

    def test_long_lines(self):
        dbv = DBVuln.from_id(124)
        self.assertEqual(dbv.description, u'A very long description for'
                                          u' Cross-Site Scripting')

    def test_long_lines_with_new_line(self):
        dbv = DBVuln.from_id(125)
        self.assertEqual(dbv.description, u'Start line 1\n'
                                          u'Start line 2\n')



#!/usr/bin/env python

from setuptools import setup, find_packages
from os.path import join, dirname


setup(
    name='vulndb',

    version='0.0.1',
    license='BSD 3-clause',
    platforms='Linux',

    description='Provides access to the vulndb information',
    long_description=open(join(dirname(__file__), 'README.md')).read(),

    author='Andres Riancho',
    author_email='andres@tagcube.io',
    url='https://github.com/vulndb/python-sdk/',

    packages=[p for p in find_packages() if p.startswith('vulndb')],
    include_package_data=True,

    install_requires=[],

    # With setuptools_git we make sure that the contents of vulndb/db/ , which
    # are non source code files, get copied too
    setup_requires=['setuptools_git >= 1.1'],
    zip_safe=False,

    # https://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Natural Language :: English',
        'Operating System :: POSIX :: Linux',
        'Operating System :: Microsoft :: Windows',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Topic :: Security'
    ],
)




import unittest
import types
import os

from vulndb import DBVuln
from vulndb.db_vuln import Reference


class TestLoadAllJSON(unittest.TestCase):
    def test_from_file(self):
        for _fname in os.listdir(DBVuln.DB_PATH):
            _file_path = os.path.join(DBVuln.DB_PATH, _fname)

            dbv = DBVuln.from_file(_file_path)

            self.assertIsInstance(dbv.title, basestring)
            self.assertIsInstance(dbv.description, basestring)
            self.assertIsInstance(dbv.id, int)
            self.assertIsInstance(dbv.severity, basestring)
            self.assertIsInstance(dbv.wasc, (types.NoneType, list))
            self.assertIsInstance(dbv.tags, (types.NoneType, list))
            self.assertIsInstance(dbv.cwe, (types.NoneType, list))
            self.assertIsInstance(dbv.owasp_top_10, (types.NoneType, dict))
            self.assertIsInstance(dbv.fix, dict)
            self.assertIsInstance(dbv.fix_effort, int)
            self.assertIsInstance(dbv.fix_guidance, basestring)

            for ref in dbv.references:
                self.assertIsInstance(ref, Reference)
